#include <config.h>

#include <glib.h>
#include <gtk/gtk.h>
#include <gdk-pixbuf/gdk-pixbuf.h>
#include <gconf/gconf-client.h>
#include <glade/glade.h>
#include <libgnome/libgnome.h>
#include <libgnome/gnome-i18n.h>
#include <libgnome/gnome-desktop-item.h>
#include <libgnomeui/libgnomeui.h>

static GnomeProgram *program;

typedef struct
{
  gchar *lock_dialog_theme;
  GConfClient *gconf_client;
  GtkListStore *list_store;
  GtkCellRenderer *theme_tree_cell_pixbuf;
  GtkTreeViewColumn *theme_tree_column_pixbuf;
  GtkTreeSelection *theme_tree_selection;
  GtkCheckButton *lock_enabled_check;
  GtkHScale *lock_delay_scale;
  GtkCheckButton *user_switch_enabled_check;
  GtkCheckButton *logout_enabled_check;
  GtkHScale *logout_delay_scale;
} MainData;

void
check_lock_dialog_themes (MainData *main_data)
{
  gchar *screensaver_dir_name = NULL;
  GDir *screensaver_dir = NULL;
  gchar *filename = NULL;
  gchar *theme_name = NULL;
  gchar *formatted_name = NULL;
  gchar *comment = NULL;
  gchar *desktop_filename = NULL;
  gchar *pixbuf_filename = NULL;
  GtkTreeIter iter = {0};
  GValue theme_value = {0};
  GValue name_value = {0};
  GValue comment_value = {0};
  GValue pixbuf_value = {0};
  GnomeDesktopItem *desktop_item = NULL;
  GdkPixbuf *src_pixbuf = NULL;
  GdkPixbuf *pixbuf = NULL;
  g_value_init (&theme_value, G_TYPE_STRING);
  g_value_init (&name_value, G_TYPE_STRING);
  g_value_init (&comment_value, G_TYPE_STRING);
  g_value_init (&pixbuf_value, g_type_from_name("GdkPixbuf"));
  screensaver_dir_name = gnome_program_locate_file (program, GNOME_FILE_DOMAIN_DATADIR, "gnome-screensaver", TRUE, NULL);
  screensaver_dir = g_dir_open (screensaver_dir_name, 0, NULL);
  filename = (gchar *) g_dir_read_name (screensaver_dir);
  while (filename != NULL)
    {
      if (g_str_has_prefix (filename, "lock-dialog-") == TRUE &&
          (g_str_has_suffix (filename, ".glade") == TRUE ||
          g_str_has_suffix (filename, ".ui") == TRUE))
        {
          if (g_str_has_suffix (filename, ".ui") == TRUE)
            {
              theme_name = g_strndup (filename + 12, g_utf8_strlen (filename + 12, -1) - 3);
            }
          else
            {
              theme_name = g_strndup (filename + 12, g_utf8_strlen (filename + 12, -1) - 6);
            }
          g_value_set_string (&theme_value, theme_name);
          gtk_list_store_append (main_data->list_store, &iter);
          gtk_list_store_set_value (main_data->list_store, &iter, 2,
                                    &theme_value);
          desktop_filename = g_strdup_printf ("%s%s%s%s", LOCK_THEME_DIR, "/lock-dialog-", theme_name, ".desktop");
          if (g_file_test (desktop_filename, G_FILE_TEST_EXISTS | G_FILE_TEST_IS_REGULAR))
            {
              desktop_item = gnome_desktop_item_new_from_file (desktop_filename, GNOME_DESKTOP_ITEM_LOAD_ONLY_IF_EXISTS, NULL);
              formatted_name = (gchar *) gnome_desktop_item_get_localestring (desktop_item, GNOME_DESKTOP_ITEM_NAME);
              comment = (gchar *) gnome_desktop_item_get_localestring (desktop_item, GNOME_DESKTOP_ITEM_COMMENT);
              g_value_set_string (&name_value, formatted_name);
              g_value_set_string (&comment_value, comment);
              gtk_list_store_set_value (main_data->list_store, &iter, 1,
                                        &name_value);
              gtk_list_store_set_value (main_data->list_store, &iter, 3,
                                        &comment_value);
            }
          else
            {
              gtk_list_store_set_value (main_data->list_store, &iter, 1,
                                        &theme_value);
              gtk_list_store_set_value (main_data->list_store, &iter, 3,
                                        &theme_value);
            }
          g_free (desktop_filename);

          pixbuf_filename = g_strdup_printf ("%s%s%s%s", LOCK_THEME_DIR, "/lock-dialog-", theme_name, ".png");
          if (g_file_test (pixbuf_filename, G_FILE_TEST_EXISTS | G_FILE_TEST_IS_REGULAR))
            {
              src_pixbuf = gdk_pixbuf_new_from_file (pixbuf_filename, NULL);
              if (src_pixbuf)
                {
                  pixbuf = gdk_pixbuf_scale_simple (src_pixbuf, 48, 48, GDK_INTERP_BILINEAR);
                  if (pixbuf)
                    {
                      g_value_set_object(&pixbuf_value, pixbuf);
                      gtk_list_store_set_value (main_data->list_store, &iter, 0,
                                                &pixbuf_value);
                    }
                  g_object_unref(src_pixbuf);
                }
            }
          g_free (pixbuf_filename);
  
          if (!g_strcmp0 (theme_name, main_data->lock_dialog_theme))
            {
              gtk_tree_selection_select_iter (GTK_TREE_SELECTION
                                              (main_data->theme_tree_selection),
                                              &iter);
            }
          g_free (theme_name);
        }
      filename = (gchar *) g_dir_read_name (screensaver_dir);
    }
  g_dir_close (screensaver_dir);
}

void
lock_theme_selection_changed (GtkTreeSelection * selection,
			      gpointer user_data)
{
  MainData *main_data = user_data;
  GtkTreeModel *model;
  GtkTreeIter iter;
  gchar *new_theme;
  if (gtk_tree_selection_get_selected (selection, &model, &iter))
    {
      gtk_tree_model_get (model, &iter, 2, &new_theme, -1);
      gconf_client_set_string (main_data->gconf_client,
			       "/apps/gnome-screensaver/lock_dialog_theme",
			       new_theme, NULL);
      g_free (new_theme);
    }
  return;
}

void
lock_enabled_check_toggled (GtkCheckButton * check, gpointer user_data)
{
  MainData *main_data = user_data;
  gboolean new_value =
    gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON
                               (main_data->lock_enabled_check));
    gconf_client_set_bool (main_data->gconf_client,
			   "/apps/gnome-screensaver/lock_enabled", new_value, NULL);
}

void
lock_delay_scale_value_changed (GtkHScale * hscale, gpointer user_data)
{
  MainData *main_data = user_data;
  gint new_value =
    gtk_range_get_value (GTK_RANGE
                               (hscale));
    gconf_client_set_int (main_data->gconf_client,
			   "/apps/gnome-screensaver/lock_delay", new_value, NULL);
}

void
user_switch_enabled_check_toggled (GtkCheckButton * check, gpointer user_data)
{
  MainData *main_data = user_data;
  gboolean new_value =
    gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON
                               (check));
    gconf_client_set_bool (main_data->gconf_client,
			   "/apps/gnome-screensaver/user_switch_enabled", new_value, NULL);
}

void
logout_enabled_check_toggled (GtkCheckButton * check, gpointer user_data)
{
  MainData *main_data = user_data;
  gboolean new_value =
    gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON
                               (check));
    gconf_client_set_bool (main_data->gconf_client,
			   "/apps/gnome-screensaver/logout_enabled", new_value, NULL);
}

void
logout_delay_scale_value_changed (GtkHScale * scale, gpointer user_data)
{
  MainData *main_data = user_data;
  gint new_value =
    gtk_range_get_value (GTK_RANGE
                               (scale));
    gconf_client_set_int (main_data->gconf_client,
			   "/apps/gnome-screensaver/logout_delay", new_value, NULL);
}

gboolean theme_tree_check_for_theme (GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *iter, gpointer user_data) {
  MainData *main_data = user_data;
  GValue value = {0};
  gchar *theme_name = NULL;
  gtk_tree_model_get_value (model, iter, 2, &value);
  theme_name = (gchar *) g_value_get_string (&value);
  if (!g_strcmp0(main_data->lock_dialog_theme, theme_name)) {
    gtk_tree_selection_select_iter(GTK_TREE_SELECTION(main_data->theme_tree_selection), iter);
    return (TRUE);
  }
  return (FALSE);
}

void
lock_dialog_theme_gconf_callback (GConfClient * client,
				  guint cnxn_id,
				  GConfEntry * entry, gpointer user_data)
{
  MainData *main_data = user_data;
  if (gconf_entry_get_value (entry) != NULL &&
      gconf_entry_get_value (entry)->type == GCONF_VALUE_STRING)
    {
      gchar *new_value =
	(gchar *) gconf_value_get_string (gconf_entry_get_value (entry));
      if (g_strcmp0 (new_value, main_data->lock_dialog_theme))
        {
          main_data->lock_dialog_theme = new_value;
          gtk_tree_model_foreach(GTK_TREE_MODEL(main_data->list_store), (GtkTreeModelForeachFunc) * theme_tree_check_for_theme, main_data);
        }
    }
}

void
lock_enabled_gconf_callback (GConfClient * client,
			     guint cnxn_id,
			     GConfEntry * entry, gpointer user_data)
{
  MainData *main_data = user_data;
  if (gconf_entry_get_value (entry) != NULL &&
      gconf_entry_get_value (entry)->type == GCONF_VALUE_BOOL)
    {
      gboolean new_value =
	gconf_value_get_bool (gconf_entry_get_value (entry));
      gboolean check_state = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON
				   (main_data->lock_enabled_check));
      if (check_state != new_value)
        {
        gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON
				   (main_data->lock_enabled_check),
				   new_value);
	}
    }
}

void
lock_delay_gconf_callback (GConfClient * client,
			   guint cnxn_id,
			   GConfEntry * entry, gpointer user_data)
{
  MainData *main_data = user_data;
  if (gconf_entry_get_value (entry) != NULL &&
      gconf_entry_get_value (entry)->type == GCONF_VALUE_INT)
    {
      gint new_value =
	gconf_value_get_int (gconf_entry_get_value (entry));
      gint check_state = gtk_range_get_value (GTK_RANGE
				   (main_data->lock_delay_scale));
      if (check_state != new_value)
        {
      gtk_range_set_value (GTK_RANGE (main_data->lock_delay_scale),
				   new_value);
	}
    }
}

void
user_switch_enabled_gconf_callback (GConfClient * client,
				    guint cnxn_id,
				    GConfEntry * entry, gpointer user_data)
{
  MainData *main_data = user_data;
  if (gconf_entry_get_value (entry) != NULL &&
      gconf_entry_get_value (entry)->type == GCONF_VALUE_BOOL)
    {
      gboolean new_value =
	gconf_value_get_bool (gconf_entry_get_value (entry));
      gboolean check_state = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON
				   (main_data->lock_enabled_check));
      if (check_state != new_value)
        {
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON
				   (main_data->user_switch_enabled_check),
				   new_value);
	}
    }
}

void
logout_enabled_gconf_callback (GConfClient * client,
			       guint cnxn_id,
			       GConfEntry * entry, gpointer user_data)
{
  MainData *main_data = user_data;
  if (gconf_entry_get_value (entry) != NULL &&
      gconf_entry_get_value (entry)->type == GCONF_VALUE_BOOL)
    {
      gboolean new_value =
	gconf_value_get_bool (gconf_entry_get_value (entry));
      gboolean check_state = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON
				   (main_data->lock_enabled_check));
      if (check_state != new_value)
        {
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON
				   (main_data->logout_enabled_check),
				   new_value);
	}
    }
}

void
logout_delay_gconf_callback (GConfClient * client,
			     guint cnxn_id,
			     GConfEntry * entry, gpointer user_data)
{
  MainData *main_data = user_data;
  if (gconf_entry_get_value (entry) != NULL &&
      gconf_entry_get_value (entry)->type == GCONF_VALUE_INT)
    {
      gint new_value =
	gconf_value_get_int (gconf_entry_get_value (entry));
      gint check_state = gtk_range_get_value (GTK_RANGE
				   (main_data->logout_delay_scale));
      if (check_state != new_value)
        {
        gtk_range_set_value (GTK_RANGE (main_data->logout_delay_scale),
				   new_value);
	}
    }
}

static void
on_destroy (GtkWidget * widget, gpointer data)
{
  gtk_main_quit ();
}

int
main (int argc, char *argv[])
{
  MainData main_data;
  gboolean lock_enabled = FALSE;
  gint lock_delay = 120;
  gboolean user_switch_enabled = FALSE;
  gboolean logout_enabled = FALSE;
  gint logout_delay = 0;
  GError *error = NULL;
  GladeXML *glade_xml;
  GtkDialog *main_dialog;
  GtkTreeView *theme_tree;
  GtkTreeViewColumn *theme_tree_column[3];
  GtkCellRenderer *theme_tree_cell_text;
  GOptionContext *option_context;

  bindtextdomain (PACKAGE, LOCALE_DIR);
  bind_textdomain_codeset (PACKAGE, "UTF-8");
  textdomain (PACKAGE);

  option_context = g_option_context_new (PACKAGE);

  program = gnome_program_init (PACKAGE, VERSION, LIBGNOMEUI_MODULE, argc, argv, GNOME_PARAM_GOPTION_CONTEXT, option_context, GNOME_PARAM_HUMAN_READABLE_NAME, _("GNOME Lock Dialog Preferences"), GNOME_PARAM_NONE);
//  gtk_init_with_args (&argc, &argv, _("GNOME Lock Dialog Preferences"), NULL, PACKAGE, &error);

  main_data.gconf_client = gconf_client_get_default ();

  gconf_client_add_dir (main_data.gconf_client, "/apps/gnome-screensaver",
			GCONF_CLIENT_PRELOAD_NONE, NULL);

  main_data.lock_dialog_theme =
    gconf_client_get_string (main_data.gconf_client,
			     "/apps/gnome-screensaver/lock_dialog_theme",
			     NULL);
  lock_enabled =
    gconf_client_get_bool (main_data.gconf_client,
			   "/apps/gnome-screensaver/lock_enabled", NULL);
  lock_delay =
    gconf_client_get_int (main_data.gconf_client,
			  "/apps/gnome-screensaver/lock_delay", NULL);
  user_switch_enabled =
    gconf_client_get_bool (main_data.gconf_client,
			   "/apps/gnome-screensaver/user_switch_enabled",
			   NULL);
  logout_enabled =
    gconf_client_get_bool (main_data.gconf_client,
			   "/apps/gnome-screensaver/logout_enabled", NULL);
  logout_delay =
    gconf_client_get_int (main_data.gconf_client,
			  "/apps/gnome-screensaver/logout_delay", NULL);

  glade_xml =
    glade_xml_new (GLADE_DIR "/gnome-lock-dialog-preferences.glade",
                   NULL, NULL);

  main_data.list_store = gtk_list_store_new (4, g_type_from_name("GdkPixbuf"), G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING);

  main_dialog = GTK_DIALOG (glade_xml_get_widget (glade_xml, "main_dialog"));
  theme_tree = GTK_TREE_VIEW (glade_xml_get_widget (glade_xml, "theme_tree"));
  main_data.lock_enabled_check =
    GTK_CHECK_BUTTON (glade_xml_get_widget (glade_xml, "lock_enabled_check"));
  main_data.lock_delay_scale =
    GTK_HSCALE (glade_xml_get_widget (glade_xml, "lock_delay_scale"));
  main_data.user_switch_enabled_check =
    GTK_CHECK_BUTTON (glade_xml_get_widget
		      (glade_xml, "user_switch_enabled_check"));
  main_data.logout_enabled_check =
    GTK_CHECK_BUTTON (glade_xml_get_widget (glade_xml, "logout_enabled_check"));
  main_data.logout_delay_scale =
    GTK_HSCALE (glade_xml_get_widget (glade_xml, "logout_delay_scale"));
  theme_tree_cell_text = gtk_cell_renderer_text_new ();
  main_data.theme_tree_cell_pixbuf = gtk_cell_renderer_pixbuf_new ();
  main_data.theme_tree_column_pixbuf =
    gtk_tree_view_column_new_with_attributes ("Preview",
					      GTK_CELL_RENDERER
					      (main_data.theme_tree_cell_pixbuf),
					      "pixbuf", 0, NULL);
  theme_tree_column[0] =
    gtk_tree_view_column_new_with_attributes ("Name",
					      GTK_CELL_RENDERER
					      (theme_tree_cell_text),
					      "text", 1, NULL);
  theme_tree_column[1] =
    gtk_tree_view_column_new_with_attributes ("Base Name",
					      GTK_CELL_RENDERER
					      (theme_tree_cell_text),
					      "text", 2, NULL);

  theme_tree_column[2] =
    gtk_tree_view_column_new_with_attributes ("Tooltip",
					      GTK_CELL_RENDERER
					      (theme_tree_cell_text),
					      "text", 3, NULL);
  main_data.theme_tree_selection =
    gtk_tree_view_get_selection (GTK_TREE_VIEW (theme_tree));

  gtk_tree_selection_set_mode (main_data.theme_tree_selection,
			       GTK_SELECTION_SINGLE);
  gtk_tree_view_append_column (GTK_TREE_VIEW (theme_tree), main_data.theme_tree_column_pixbuf);
  gtk_tree_view_append_column (GTK_TREE_VIEW (theme_tree), theme_tree_column[0]);
  gtk_tree_view_append_column (GTK_TREE_VIEW (theme_tree), theme_tree_column[1]);
  gtk_tree_view_append_column (GTK_TREE_VIEW (theme_tree), theme_tree_column[2]);
  gtk_tree_view_column_set_visible(main_data.theme_tree_column_pixbuf, TRUE);
  gtk_tree_view_column_set_visible(theme_tree_column[0], TRUE);
  gtk_tree_view_column_set_visible(theme_tree_column[1], FALSE);
  gtk_tree_view_column_set_visible(theme_tree_column[2], FALSE);
  gtk_tree_view_set_tooltip_column(GTK_TREE_VIEW (theme_tree), 3);

  gtk_tree_view_set_model (GTK_TREE_VIEW (theme_tree),
			   GTK_TREE_MODEL (main_data.list_store));

  check_lock_dialog_themes (&main_data);

  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON
			       (main_data.lock_enabled_check), lock_enabled);
  gtk_range_set_value (GTK_RANGE (main_data.lock_delay_scale), lock_delay);
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON
			       (main_data.user_switch_enabled_check),
			       user_switch_enabled);
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON
			       (main_data.logout_enabled_check),
			       logout_enabled);
  gtk_range_set_value (GTK_RANGE (main_data.logout_delay_scale),
		       logout_delay);

  gconf_client_notify_add (main_data.gconf_client,
			   "/apps/gnome-screensaver/lock_dialog_theme",
			   lock_dialog_theme_gconf_callback, &main_data, NULL,
			   NULL);
  gconf_client_notify_add (main_data.gconf_client,
			   "/apps/gnome-screensaver/lock_enabled",
			   lock_enabled_gconf_callback, &main_data, NULL,
			   NULL);
  gconf_client_notify_add (main_data.gconf_client,
			   "/apps/gnome-screensaver/lock_delay",
			   lock_delay_gconf_callback, &main_data, NULL, NULL);
  gconf_client_notify_add (main_data.gconf_client,
			   "/apps/gnome-screensaver/user_switch_enabled",
			   user_switch_enabled_gconf_callback, &main_data,
			   NULL, NULL);
  gconf_client_notify_add (main_data.gconf_client,
			   "/apps/gnome-screensaver/logout_enabled",
			   logout_enabled_gconf_callback, &main_data, NULL,
			   NULL);
  gconf_client_notify_add (main_data.gconf_client,
			   "/apps/gnome-screensaver/logout_delay",
			   logout_delay_gconf_callback, &main_data, NULL,
			   NULL);


  glade_xml_signal_connect (glade_xml, "on_destroy", G_CALLBACK (on_destroy));
  g_signal_connect (G_OBJECT (main_data.theme_tree_selection), "changed",
		    G_CALLBACK (lock_theme_selection_changed), &main_data);
  g_signal_connect (G_OBJECT (main_data.lock_enabled_check), "toggled",
		    G_CALLBACK (lock_enabled_check_toggled), &main_data);
  g_signal_connect (G_OBJECT (main_data.lock_delay_scale), "value_changed",
		    G_CALLBACK (lock_delay_scale_value_changed), &main_data);
  g_signal_connect (G_OBJECT (main_data.user_switch_enabled_check), "toggled",
		    G_CALLBACK (user_switch_enabled_check_toggled), &main_data);
  g_signal_connect (G_OBJECT (main_data.logout_enabled_check), "toggled",
		    G_CALLBACK (logout_enabled_check_toggled), &main_data);
  g_signal_connect (G_OBJECT (main_data.logout_delay_scale), "value_changed",
		    G_CALLBACK (logout_delay_scale_value_changed),
		    &main_data);

  gtk_main ();

  return 0;
}
